/***************************************************************************
                          feld.cpp  -  description
                             -------------------
    begin                : Die Feb 18 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de

    This file is part of Brandubh, a game that was develop for BWInf2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "feld.h"

#ifdef DEBUG
	#include <iostream>
	using namespace std;
#endif

Feld::Feld(int nr, int (*feldGewaehlt)(int nr)){
  this->nr = nr;
  this->feldGewaehlt = feldGewaehlt;
	
	// feldgre initialisieren
	this->pxHeight = FELD_HOEHE;
	this->pxWidth = FELD_BREITE;
	this->setColorMask(0,0,0,0);

	this->ausgewaehlt = 0; // = false;
}


Feld::~Feld(){
	// der destructor von SGUI_Object reicht vollkommen
}


/** Gibt zurck, wer auf dem Feld steht.
NIEMAND
SCHWARZ
WEISS oder
KOENIG */
int Feld::wer(void){
	return this->besetzt;
}


/** Setzt eine Figur auf das Feld. Mgliche werte sind:
NIEMAND
SCHWARZ
WEISS
KOENIG */
int Feld::setze(int wen){
	// range-check
	if(wen < NIEMAND || wen > KOENIG)
		return -1;

	// setzte figur auf feld
	this->besetzt = wen;

	// jetzt muss noch das aussehen aktualisiert werden
	// schaue nach, ob schon ein surface existiert, auf den man "malen" kann
	if(this->surface == NULL)
	{
		// schade, dann schau mal noch, ob schon eine ColorMask und die zum malen bentigten vorlagen existieren
		if(this->colorMask.Rmask == 0 || this->feld == NULL || this->schwarz == NULL || this->weiss == NULL || this->koenig == 0)
		{
			// das feld ist ja noch gar nicht richtig initialisiert
#ifdef DEBUG
			cout << "Das Feld wurde noch nicht fertig initialisiert. Malen abgebrochen." << endl;
#endif
			return -2;
		}

		// dann mssen wir uns mal einen Surface machen
		this->surface = SDL_CreateRGBSurface(SDL_SRCALPHA, this->pxWidth, this->pxHeight, 32, this->colorMask.Rmask, this->colorMask.Gmask, this->colorMask.Bmask, this->colorMask.Amask);
		// schau nach, ob erfolgreich
		if(this->surface == NULL)
		{
			#ifdef DEBUG
			cout << "Es konnte kein SDL_Surface erstellt werden: " << SDL_GetError() << endl;
			#endif
			return -3;
		}

		// ansonsten msste jetzt alles klar sein
	}

	// Die malrechtecke
	SDL_Rect src, dest;
	src.x = 0;
	src.y = 0;

	dest.x = 0;
	dest.y = 0;
	dest.w = FELD_BREITE;
	dest.h = FELD_HOEHE;

	// wenn das feld angewhlt ist, benutze das ausgewhlte feld, sonst male ein normales feld als hintergrund
	// um flexibler zu sein wird hier ausgewhlt, und nicht focused verwendet
	// dadurch wird automatisch eine vorher vielleicht dort vorhandene figur bermalt
	if(this->ausgewaehlt)
	{
		if(this->feld_markiert == NULL)
			return -1;
		src.w = this->feld_markiert->w;
		src.h = this->feld_markiert->h;

		SDL_BlitSurface(this->feld_markiert, &src, this->surface, &dest);
	}
	else
	{
		src.w = this->feld->w;
		src.h = this->feld->h;

		SDL_BlitSurface(this->feld, &src, this->surface, &dest);
	}

	// fge das thronfeld-ding falls notwendig hinzu
	if(this->nr == SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
	{
		src.w = this->feld->w;
		src.h = this->feld->h;

		SDL_BlitSurface(this->thron, &src, this->surface, &dest);
	}

	// jetzt male, wenn notwendig noch die figur
	switch(this->besetzt)
	{
	case SCHWARZ:
		src.w = this->schwarz->w;
		src.h = this->schwarz->w;

		SDL_BlitSurface(this->schwarz, &src, this->surface, &dest);
		break;
		
	case WEISS:
		src.w = this->weiss->w;
		src.h = this->weiss->w;

		SDL_BlitSurface(this->weiss, &src, this->surface, &dest);
		break;

	case KOENIG:
		src.w = this->koenig->w;
		src.h = this->koenig->w;

		SDL_BlitSurface(this->koenig, &src, this->surface, &dest);
		break;
	// ansonsten bleibt das feld einfach, wie es ist
	}

	// Wert und aussehen wurden upgedated und sind beim nchsten malen korrekt.
	return 42;
}
/** Sorgt dafr das das Feld, solange die Maustaste gedrckt ist, schn markiert erschein, und sorgt dafr, dass feldGewaehlt() bei einem click aufgerufen wird */
int Feld::message(SDL_Event * event){
	if(event->type == SDL_MOUSEBUTTONDOWN)
	{
		// das feld ist angewhlt, setzt den wert auf wahr
		this->ausgewaehlt++;
		// male das feld nue
		this->setze(this->besetzt);

		// rufe feldGewaehlt auf
		return this->feldGewaehlt(nr);
	}
	if(event->type == SDL_MOUSEBUTTONUP)
	{
		this->ausgewaehlt--; // wurde es nur durch die maus ausgewhlt, so gibt 1 - 1 = false. wurde es auch noch von auerhalb markiert so gibt 2 - 1 = true
		// male feld neu
		this->setze(this->besetzt);
	}

	return 42;
}
