/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Don Feb 13 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
    
    This file is part of Brandubh, a game that was develop for BWInf2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// The main Gamefile

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string>
#include <iostream>
using namespace std;

#include "sgui/sgui_window.h"


#include "config.h"
#include "feld.h"
#include "display.h"
#include "spiel.h"
#include "help.h"
#include "speichern.h"

// globale Funktionen.

/** starte das spiel von vorne */
void restart();

/** wenn ein 2spielermodus ausgewhlt wurde */
void initZweiSpieler();
/** hole die entsprechenden namen aus dem dialog, und leite die nchsten schritte ein */
void weissEingegeben();
void schwarzEingegeben();

// spiel speichern/laden
void lade();
void speicher();
void ladeDatei();
void speicherDatei();



// globale Variablen

// mgliche spielarten
#define ZWEI_SPIELER 2
int spieltyp;

// Namen der Spieler "COMPUTER" fr Computerspieler
string nameWeiss = "Spieler1";
string nameSchwarz = "Spieler2";

int main(int argc, char* argv[])
{
	// Fenster, in dem alles dargestellt wird
	Fenster.setSize(800, 600);
	Fenster.setBackgroundColor(0, 0, 0);
	Fenster.setName("Brandubh 0.1W");
	Fenster.setIcon(APP_ICON);

	if(initDialoge(feldGewaehlt, initZweiSpieler, restart, zeigeZuege, lade, speicher) < 0)
	{
		cout << "So hat das Spiel wenig sinn :(" << endl;
		return -1;
	}

	SDL_WM_SetCaption("Brandubh", NULL);

	Fenster.addObject(Spieltyp);
	Fenster.addObject(Hilfe);
	Fenster.addObject(Menue);

	// hilfe zum spieltyp
	zeigeHilfe(HELP_SPIELTYP);

	// bringe das spiel schon mal in startaufstellung
	startaufstellung();

  // zeichne fenster, damit alles gesehen wird
  Fenster.draw();
	
	// Verarbeite die Ereignisse
	SDL_Event ereignis;
	bool fertig = false;
	while(!fertig)
	{
		// hole die ereignisse ab
		ereignis = Fenster.WaitEvent();

		// schaue nach, ob das Programm beendet ist
		if(ereignis.type == SDL_QUIT)
			fertig = true;

		// zeichne fenster neu
		Fenster.draw(); 
	};

  unloadDialoge();
	
	return 42;
};








void initZweiSpieler()
{
	// Entferne die Spieltyp-auswahl
	Fenster.removeObject(Spieltyp);

	spieltyp = ZWEI_SPIELER;

	// zeige namenseingabe-hilfe
	zeigeHilfe(HELP_NAMEN);

	// Frage nach den Spielernamen
	// dazu Name-Dialog neue initialisieren
	NameText->setText("Bitte den Name des weissen Spielers eingeben:");
	NameInput->setText("");
	NameOK->setAction(weissEingegeben);

	// focusiere die Eingabe auf den Text
	Name->setFocus(NameInput);

	Fenster.addObject(Name);
	Fenster.setFocus(Name);

	return;
}


void weissEingegeben()
{
	nameWeiss = NameInput->getText();

	// nachschauen, ob der Name auch wirklich eingegeben wurde
	if(nameWeiss == "")
	{
		// ansonsten Namen nochmal erfragen
		// dazu Name-Dialog neue initialisieren
		NameText->setText("Bitte den Name des weissen Spielers eingeben:");
		NameInput->setText("");
		NameOK->setAction(weissEingegeben);

		// focusiere die Eingabe auf den Text
		Name->setFocus(NameInput);

		Fenster.setFocus(Name);

		// und auf ereignisse warten
		return;
	}

	
	// wenn zweispielermodus, dann nach schwarz fragen
	if(spieltyp == ZWEI_SPIELER)
	{
		NameText->setText("Bitte den Namen des schwarzen Spielers eingeben:");
		NameInput->setText("");
		NameOK->setAction(schwarzEingegeben);

		Name->setFocus(NameInput);
		Fenster.setFocus(Name);
		return;
	}

	// ansonsten blende das fenster aus und fertig
	Fenster.removeObject(Name);

	starteSpiel();
	
	return;
};

void schwarzEingegeben()
{
	nameSchwarz = NameInput->getText();

	// nachschauen, ob der name wirklich eigegeben wurde
	if(nameSchwarz == "")
	{
		NameText->setText("Bitte den Namen des schwarzen Spielers eingeben:");
		NameInput->setText("");
		NameOK->setAction(schwarzEingegeben);

		Name->setFocus(NameInput);
		Fenster.setFocus(Name);
		
		return;
	}	

	// blende fenster aus
	Fenster.removeObject(Name);

	// das spiel kann gestartet werden
	starteSpiel();

	return;
};

void restart()
{
	Fenster.removeObject(GameOver);

	// es kann nicht gespeichert werden
	Spieltyp->removeObject(SpeicherKnopf);
	Spieltyp->removeObject(CancelOptions);

	Fenster.addObject(Spieltyp);

	// zeige coole hilfe an
	zeigeHilfe(HELP_SPIELTYP);

	return;
}


void speicher()
{
	DateiText->setText("Wie soll das gespeicherte Spiel heissen?");
	DateiInput->setText("");
  DateiOK->setAction(speicherDatei);

	Fenster.addObject(DateiDialog);
	Fenster.setFocus(DateiDialog);
	DateiDialog->setFocus(DateiInput);

	zeigeHilfe(HELP_SAVE);

}

void lade()
{
	DateiText->setText("Bitte den Dateinamen des Spieles eingeben:");
	DateiInput->setText("");
	DateiDialog->setFocus(DateiInput);
	DateiOK->setAction(ladeDatei);

	Fenster.addObject(DateiDialog);
	Fenster.setFocus(DateiDialog);
	DateiDialog->setFocus(DateiInput);

	zeigeHilfe(HELP_LOAD);
}

void speicherDatei()
{
	string sDatei = DateiInput->getText();

	Fenster.removeObject(DateiDialog);

	if(sDatei == "")
	{
		speicher();
		return;
	}

	saveToFile(sDatei);

	Fenster.addObject(Spieltyp);

	// es geht in's optionenmenue zurck, also
	zeigeHilfe(HELP_SPIELTYP);
	
	return;
}

void ladeDatei()
{
	string sDatei = DateiInput->getText();

	Fenster.removeObject(DateiDialog);


	if(sDatei == "")
	{
		lade();
		return;
	}

	if(restoreFromFile(sDatei) < 0)
		lade();
	else
		// das spiel geht weiter
		zeigeHilfe(HELP_SPIELANFANG);

	return;
}
