/***************************************************************************
                          sgui_object.h  -  description
                             -------------------
    begin                : Mon Feb 10 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SGUI_OBJECT_H
#define SGUI_OBJECT_H


/**A basic SGUI_Object that all SGUI_Object will be derived from
  *@author Matthias Bach
  */

#include <SDL/SDL.h>

#define SGUI_NO_EVENT 0

#define SGUI_BUTTON 1
#define SGUI_TEXTFIELD 5
#define SGUI_MENU 10


struct SGUI_ColorMask
{
	Uint32 Rmask, Gmask, Bmask, Amask;
};




class SGUI_Object {
public: 
	SGUI_Object();
	virtual ~SGUI_Object();
  /** This renders the Object to the returned Surface */
  virtual SDL_Surface * draw();
  /** Returns the width of the object in pixels */
  int width();
  /** Returns the height of the object in pixels */
  int height();
  /** Returns the Type of the object */
  int getType();
	/** Sends an event to the Object */
  virtual int message(SDL_Event * event);
  /** Sets the Colormask of the object */
  int setColorMask(Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
protected: // Protected attributes
  /** The objects surface that will be returned when drawn */
  SDL_Surface * surface;

  /** The type of the Object */
  int type;
  /** The width of the object in pixels */
  int pxWidth;
  /** The height of the object in pixels */
  int pxHeight;
  /** The colormask the object should have */
  SGUI_ColorMask colorMask;


public: // Public attributes
  /** If true, the object should be seen on the screen */
  bool visible;
  /** The object's position on the x-Axis */
  int x;
  /** The object's position on the y-Axis */
  int y;
  /** If this is set, the object is supposed to have user focus */
  bool focused;
private: // Private attributes

};

#endif
