/***************************************************************************
                          sgui_window.h  -  description
                             -------------------
    begin                : Sam Feb 8 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SGUI_WINDOW_H
#define SGUI_WINDOW_H

/**A basic window that provices space for menus, frames and place to draw stuff
  *@author Matthias Bach
  */

#include <vector>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "sgui_object.h" // needed for object acces (drawing, messages)


// include all others, so app wil only have to include sgui_window.h
#include "sgui_menu.h"
#include "sgui_button.h"
#include "sgui_textfield.h"
#include "sgui_dialog.h"

class SGUI_Window {
public: 
	SGUI_Window();
	~SGUI_Window();
  /** This will blit the passed SDL_Surface to the Window (so it won't kill any other stuff) */
  int blit(SDL_Surface * src, SDL_Rect * srcRect, SDL_Rect * destRect);
  /** Writes the given text to the screen using a standard Font */
  int write(std::string text, int x, int y);
  /** Returns the width of the window that can acutally been drawn to
 */
  int width();
  /** Returns the height in px of the window that can acutally be drawn to */
  int height();
  /** Sets the size of the Window
Return negative value on error */
  int setSize(int x, int y);
  /** Puts the window on the Screen. */
  int show();
  /** Draws the Window (including all buttons) */
  int draw();
  /** Wait's for the next Event, remotes it to the hitten objects window and returns it's code, or the Event itself if it was not handled */
  SDL_Event WaitEvent();
  /** Check whether there was an event, and handles if nesecarry */
  SDL_Event PeekEvent();
  /** An Object is added to the window (and the objects-vector). This will also be initialized to the windows display-format etc. */
  int addObject(SGUI_Object * object);
  /** Removes an Object from the SGUI_Window, but does not delete it! */
  int removeObject(SGUI_Object * object);
  /** Sets the Color of the background by putting the values into the Background-Color Struct
      Changes won't take effekt until after the next draw!*/
  int setBackgroundColor(Uint8 red, Uint8 green, Uint8 blue);
  /** Allows to set the input focus on a certain object in the dialog
returns -1 if the given object does not exist in the dialog */
  int setFocus(SGUI_Object * object);

  /** Allows to specify an icon to be used for the window
return -1 if passed an empty string and -2 if window is already displayed */
  int setIcon(std::string iconName);

  /** Allows to specify a name to be used for the window
return -1 if passed an empty string and -2 if window is already displayed */
  int setName(std::string windowName);
  
public: // Public attributes

  /** Menu's are only drawn if this is true */
  bool showMenus;

protected: // Protected attributes
  /** The acutall screen that everything is drawn to */
  SDL_Surface * screen;
  /** Stores the Color the Background is cleared with */
  SDL_Color backgroundColor;

	std::vector <SGUI_Object *> objects;

private: // Private attributes
  /** The actual height of the window */
  int realHeight;
  /** The width of the window */
  int realWidth;
  /** The Font used in this Window */
  TTF_Font * font;
  /** The Color of text drawn to the window */
  SDL_Color textColor;
  /** If this is true, the window is shown */
  int visible;
  /** The current object */
  int focusedObject;
  /** The name of the file containing the icon */
  std::string iconName;
  /** Pointer to the used icon (if any) for proper freeing on exit */
  SDL_Surface* icon;
  /** The name to be used for the window */
  std::string windowName;

private: // Private methods
  /** No descriptions */
  int handleEvents(SDL_Event * event);
};

#endif
