/***************************************************************************
                          snapshots.cpp  -  description
                             -------------------
    begin                : Fre Feb 21 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de

    This file is part of Brandubh, a game that was develop for BWInf2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "snapshots.h"

#include "display.h"
#include "spiel.h"

#include <fstream> // fr auslesen der hilfedateien
using namespace std;

vector<int*> Snapshots;

bool init = false;

void deleteSnapshots()
{
	for(unsigned int i = 0; i < Snapshots.size(); i++)
	{
		delete[] Snapshots[i];
	}
	Snapshots.clear();
}

void makeSnapshot()
{
	int * Snapshot = getSnapshot();

	// fge Snappshus ins arichv
	Snapshots.push_back(Snapshot);

	// das wars
	return;
}

int* getSnapshot()
{
	if(init == false)
	{
		// stelle sicher, das der speicher aufgerumt wird
		atexit(deleteSnapshots);
		init = true;
	}

	int * Snapshot = new int[SPIELFELD_SPALTEN * SPIELFELD_REIHEN];

	// kopiere Spielfeld daten
	for(int i = 0; i < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; i++)
		Snapshot[i] = Felder[i]->wer();

	return Snapshot;
}


int compareWithSnapshots()
{
	int found = 0;
	bool correct;
	

	// vergleiche die vorhandenen snapshots mit dem aktuellen feld
	// es wird ausgehend vom vorletzten immer nur mit jedem zweiten
	// verglichen, so dass nur die snaphots drankommen, bei der auch der aktuelle
	// spieler am zug war
	// dies setzt vorraus, dass der aktuelle zug noch nicht archiviert ist
	for(int snapshot = Snapshots.size() - 2; snapshot > -1; snapshot -= 2)
	{
		correct = true;
		// schaue, ob es ein feld des aktuellen shots gibt, das nicht mit dem aktuellen feld berinstimmt
		for(int i = 0; i < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; i++)
			if(Snapshots[snapshot][i] != Felder[i]->wer())
				correct = false;

		// stimmten alle felder berein, so ist die situation schon mal aufgetreten
		if(correct == true)
			found++;
	}
	
	return found;
};


int restoreFromSnapshot(int * Snapshot)
{
	// kopiere Spielfeld daten
	for(int i = 0; i < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; i++)
		Felder[i]->setze(Snapshot[i]);

	return 42;
}


/** Die 48er verschiebung produziert human-readable-files */

int saveToFile(string sDatei)
{
  // unterverzeichnis zum Dateinamen hinzufgen
	sDatei = "saves/" + sDatei;
	
	// das spiel kann nur gespeichert werden, wenn ein spieler am zug wre
	char wer = dran();
	if(wer != SCHWARZ && wer != WEISS)
		return -1;

	// ansonsten: datei ffnen
	ofstream Datei(sDatei.c_str());

	if(!Datei)
		return -2;

	// schreibe, wer gerade dran ist
	// smtliches schreiben in die datei wird ber put() gemacht, damit immer genau ein char eingelesen wird.
	Datei.put(wer + 48);
	Datei.put('\n');


	// speicher die Namen der Spieler (mit terminierender 1)
  Datei << nameWeiss << (char) 1 << '\n';
  Datei << nameSchwarz << (char) 1 << '\n';
  	

	// schreibe alle screenshots, der letzte screenshot ist der aktuelle spielstand
	for(unsigned int snapshot = 0; snapshot < Snapshots.size(); snapshot++)
	{
		for(int feld = 0; feld < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; feld++)
			Datei.put(Snapshots[snapshot][feld] + 48);
		Datei.put('\n');
	}

	Datei.close();

	return 42;
}

int restoreFromFile(string sDatei)
{
	// unterverzeichnis zum Dateinamen hinzufgen
	sDatei = "saves/" + sDatei;

	char ch; // all purpose char um den input zwischenzuspeichern

	// ffne die datei
	ifstream Datei(sDatei.c_str());

	if(!Datei)
		return -2;

		
	// lese ein wer am zug ist
	Datei.get(ch);
	ch -= 48;
	if(ch != SCHWARZ && ch != WEISS) // kein savegame
		return -3;

	setDran(ch);

	// berpspringe linebreak
	Datei.get(ch);


	// lese die Namen der Spieler ein
	// Der Spielername wir 0 gesetzt.
	// Anschliesend, immer ein zeichen aus der Datei geholt, solange dieses nicht
	// den wert 1 hat, wird es zum namen hinzugefgt,
	// ansonsten in die nchste zeile gesprungen und der nchste name eingelesen
	// bzw. das laden fortgesetzt.
	bool done = false;
	nameWeiss = "";

	while(!done)
	{
		Datei.get(ch);
		if(ch == 1)
			done = true;
		else
			nameWeiss += ch;
	}
	Datei.get(ch); // nchste zeile

	// nchster spieler
	done = false;
	nameSchwarz = "";

	while(!done)
	{
		Datei.get(ch);
		if(ch == 1)
			done = true;
		else
			nameSchwarz += ch;
	}

	Datei.get(ch); // nchste Zeile

	// zeige, wer am zug ist
	if(dran() == 1)
		// zeige das weissamZug ist
		zugNachricht->setText("Am Zug:\n" + nameSchwarz);
	else
		zugNachricht->setText("Am Zug:\n" + nameWeiss);


	// in jeder der nchsten zeile steht ein snapshot

	// lsche erst mal die alten
	deleteSnapshots();

	// lese die snapshots ein, der letzte snapshot ist der aktuelle spielstand
	while(!Datei.eof())
	{
		// speicher fr einen snapshot
		int * snapshot = new int[SPIELFELD_SPALTEN * SPIELFELD_REIHEN];

		// eine zeile hineinkopieren
		for(int feld = 0; feld < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; feld++)
		{
			Datei.get(ch);
			snapshot[feld] = ch - 48;
		}

		// snapshot "abspeichern" (registrieren)
		Snapshots.push_back(snapshot);

		// linebreak
		Datei.get(ch);
	}

	// der letzte snapshot enthlt mll
	Snapshots.pop_back();

	Datei.close();

	// spielstand in's feld kopieren
	restoreFromSnapshot(Snapshots[Snapshots.size() - 1]);

	// ab jetzt kann gespeichert und in spiel zurckgekehrt werden
	// dummerweise kann nur an sichbaren objekte etwas hinzugefgt werden
	Spieltyp->addObject(SpeicherKnopf);
	Spieltyp->addObject(CancelOptions);
	// jetzt soll ja weitergespielt werden, also optionen verstecken
	Fenster.removeObject(Spieltyp);

	return 42;	
}

