/***************************************************************************
                          spiel.cpp  -  description
                             -------------------
    begin                : Don Feb 20 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de

    This file is part of Brandubh, a game that was develop for BWInf2002
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "spiel.h"
#include "display.h"
#include "feld.h"
#include "zug.h"
#include "snapshots.h"
#include "help.h"

#include <vector>
#include <string>

using namespace std;

#ifdef DEBUG
	#include <iostream>
#endif
		


// variablen

// anfangs ist noch niemand dran (erst wenn das spiel gestartet ist)
char amZug = NIEMAND;

// pausezeugs
char vorPause = NIEMAND;
char pausiert = false;
int vorPauseGewaehlt;

// auch ist noch keine figur ausgewhlt
int gewaehlteFigur = -1;
// zug-geschichte
vector<Zug> Historie;
// Wie viele Zge?
unsigned long zuege;








// "private" funktionen der Spiel-"Klasse"
// logt den zug auch
int ziehe(int von, int nach);

// fhrt den zug nur in der theorie durch, merkt ihn
// sich aber nicht, es sei denn log ist true
int ziehe(int von, int nach, bool log);


// berprft die aktuelle figur auf siegbedingungen, und schlgt figuren
// rckgabe normal 0, bei 1 hat wei, und bei 2 schwarz gewonnen
int schlagUndSiege(int nr);

// berprft, ob der knig eingekreist ist (auf dem feld nr);
bool fangeKoenig(int nr);

void schwarzGewinnt();

// schaut ob noch ein zug mglich ist
bool zugMoeglich(int wer);



// funktionen

void startaufstellung(void)
{
  int zeile, spalte;
	for(unsigned int i = 0; i < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; i++)
	{
		zeile = i / SPIELFELD_SPALTEN;
		spalte = i % SPIELFELD_SPALTEN;

		if(i == SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
		{
			// setze knig
			Felder[i]->setze(KOENIG);
		}
		// in der ersten und letzen zeile mssen fast alle felder gesetzt werden
		else if(zeile == 0 || zeile == SPIELFELD_REIHEN - 1)
		{
			if(spalte == 0 || spalte == SPIELFELD_SPALTEN / 2 || spalte == SPIELFELD_SPALTEN - 1)
      	Felder[i]->setze(NIEMAND);
   		else
     		Felder[i]->setze(SCHWARZ);
  	}
  	// in der mittleren spalte wird fast berall wei gesetzt, bis auf rand und knig
  	else if(zeile == SPIELFELD_REIHEN / 2)
  	{
			if(spalte == 0 || spalte == SPIELFELD_SPALTEN - 1)
				Felder[i]->setze(NIEMAND);
			else if(spalte != SPIELFELD_SPALTEN / 2)
				Felder[i]->setze(WEISS);
		}
		// in allen anderen zeilen 1 und letzes schwarz, mitte weiss, sonst leer
		else
		{
			if(spalte == 0 || spalte == SPIELFELD_SPALTEN - 1)
				Felder[i]->setze(SCHWARZ);
			else if(spalte == SPIELFELD_SPALTEN / 2)
				Felder[i]->setze(WEISS);
			else
				Felder[i]->setze(NIEMAND);
		}
	}

	return;
}


void starteSpiel(void)
{
	// weiss fngt an
	amZug = SCHWARZ;

	// zeige das weissamZug ist
	zugNachricht->setText("Am Zug:\n" + nameSchwarz);

	// sorge dafr, dass noch keine snapschsse existieren
	// das spiel knnte ja schon einmal gespielt worden sein
	Historie.clear();
	deleteSnapshots();
	zuege = 0;
	
	// sorge dafr, dass die figuren richtig stehen
	startaufstellung();
	makeSnapshot();

	// hilfetext ber spielanfang
	zeigeHilfe(HELP_SPIELANFANG);


	// ab jetzt kann gespeichert und in spiel zurckgekehrt werden
	// dummerweise kann nur an sichbaren objekte etwas hinzugefgt werden
	Fenster.addObject(Spieltyp);
	Spieltyp->addObject(SpeicherKnopf);
	Spieltyp->addObject(CancelOptions);
	Fenster.removeObject(Spieltyp);

	return;
}


int feldGewaehlt(int nr)
{
	if(amZug == NIEMAND)
	{
		// es findet momentan kein spiel statt. Es hat nichts zu geschehen
		return 1;
	}

  // wenn es das ausgewhlt feld ist, feld abwhlen
  if(nr == gewaehlteFigur)
  {
  	Felder[nr]->ausgewaehlt--;
  	// figur wird sowieso neu gerender wenn maustaste losgelassen wird
  	gewaehlteFigur = -1;
  }
	
	// wenn es eine weise figur ist
	else if(Felder[nr]->wer() == WEISS || Felder[nr]->wer() == KOENIG)
	{
		// schwarz kann dieses feld weder auswhlen, noch eine figur hierher-schicken
		if(amZug == SCHWARZ)
		{
			// wenn eine figur gewhlt ist, hilfetext, dass sie nicht hierher kann
			if(gewaehlteFigur > -1)
			{
				zeigeHilfe(HELP_BESETZT);
				//blabla
			}
			// ansonsten einfach ignorieren
			return 1;
		}
		// if abfrage kann wegfallen, da der dritte mgliche fall schon abgefangen wurde

		// ist schon eine figur gewhlt, so deselektiere sie
		if(gewaehlteFigur > -1)
		{
			Felder[gewaehlteFigur]->ausgewaehlt--;
			Felder[gewaehlteFigur]->setze(Felder[gewaehlteFigur]->wer());
		}

		// whle dieser figur aus
    gewaehlteFigur = nr;
		Felder[nr]->ausgewaehlt++;
		// neu rendern kann gelassen werden, da die figur sowieso schon markiert angezeigt wird;
	}

	// wenn es eine schwarze figur ist
	else if(Felder[nr]->wer() == SCHWARZ)
	{
		// weiss kan mit diesem Feld nichts anfangen
		if(amZug == WEISS)
		{
			// wenn eine figur gewhlt ist, hilfetext, dass sie nicht hierher kann
			if(gewaehlteFigur > -1)
			{
				//blabla
				zeigeHilfe(HELP_BESETZT);
			}
			// ansonsten einfach ignorieren
			return 1;
		}

		// ist schon eine figur gewhlt, so deselektiere sie
		if(gewaehlteFigur > -1)
		{
			Felder[gewaehlteFigur]->ausgewaehlt--;
			Felder[gewaehlteFigur]->setze(Felder[gewaehlteFigur]->wer());
		}
		
		// whle dieser figur aus
    gewaehlteFigur = nr;
		Felder[nr]->ausgewaehlt++;
		// neu rendern kann gelassen werden, da die figur sowieso schon markiert angezeigt wird
	}

	// wenn es ein leeres feld ist , und eine figur ausgewhlt ist (sonst geschieht nichts
	else if(gewaehlteFigur > -1)
	{
		// versuche dorthin zu hiehen
		if(ziehe(gewaehlteFigur, nr) > -1)
		{
			// war das erfolgreiche, deselektiere die alte figur
			Felder[gewaehlteFigur]->ausgewaehlt--;
			Felder[gewaehlteFigur]->setze(Felder[gewaehlteFigur]->wer());
			// es ist natrlich kein feld mehr gewhlt
			gewaehlteFigur = -1;
		}

		// sonst  passiert nichts, die alte figur bleib ausgewhlt (hier wre ein fieser kommentar cool ;)
	}
	
	return 42;
}


int ziehe(int von, int nach)
{
	return ziehe(von, nach, false);
}


int ziehe(int von, int nach, bool test)
{
	// jetzt kommt ein haufen schn hsslicher spagethi-code, but it works ;)
  // berprfe, ob der zug mglich ist (ob die felder frei sind)

  // the all too know i =)
  int i;

  int wer = Felder[von]->wer();

  // schaue, dass nur der knig den thron besetzt
  if(nach == SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2 && wer != KOENIG)
  {
		#ifdef DEBUG
			cout << "Hier darf nur der Knig hin" << endl;
		#endif
		if(!test) zeigeHilfe(HELP_THRON);
		// nur der knig darf auf den thron
		return -3;
	}


  if(nach - von >= SPIELFELD_SPALTEN) // bewegung nach unten
  {
		if(nach % SPIELFELD_SPALTEN != von % SPIELFELD_SPALTEN)
		{
			#ifdef DEBUG
			cout << "der zug ist schief" << endl;
			#endif

			if(!test) zeigeHilfe(HELP_SCHIEF);
		
			// der zug ist schief!
			return -1;
		}
    		
  	for(i = von + SPIELFELD_SPALTEN; i <= nach; i += SPIELFELD_SPALTEN)
  	{
			// wenn das feld schon besetzt ist, so drucke coolen kommentar, dass figuren nicht bersprungen werden
			// knnen, und das nur eine figur auf einem feld stehen kann,
			if(Felder[i]->wer() != NIEMAND)
			{
				#ifdef DEBUG
					cout << "Hier steht jemand im Weg" << endl;
				#endif

				if(!test) zeigeHilfe(HELP_BESETZT);
		
				// es geht nicht
				return -2;
			}
		}
	}
	else if(von - nach >= SPIELFELD_SPALTEN) // bewegung nach oben
	{
		if(nach % SPIELFELD_SPALTEN != von % SPIELFELD_SPALTEN)
		{
			#ifdef DEBUG
			cout << "der zug ist schief" << endl;
			#endif

			if(!test) zeigeHilfe(HELP_SCHIEF);
		
			// der zug ist schief!
			return -1;
		}

  	for(i = von - SPIELFELD_SPALTEN; i >= nach; i -= SPIELFELD_SPALTEN)
  	{
			// wenn das feld schon besetzt ist, so drucke coolen kommentar, dass figuren nicht bersprungen werden
			// knnen, und das nur eine figur auf einem feld stehen kann,
			if(Felder[i]->wer() != NIEMAND)
			{
				#ifdef DEBUG
					cout << "Hier steht jemand im Weg" << endl;
				#endif

				if(!test) zeigeHilfe(HELP_BESETZT);

				// es geht nicht
				return -2;
			}
		}
	}
	else if(nach - von > 0) // bewegung nach rechts
	{
		for(i = von + 1; i <= nach; i++)
  	{
			// aufpassen, dass man nicht von rechts nach links luft
			if(i % SPIELFELD_SPALTEN == 0)
			{
				#ifdef DEBUG
					cout << "der zug ist schief" << endl;
				#endif

				if(!test) zeigeHilfe(HELP_SCHIEF);
		
				// der zug ist schief
				return -1;
			}
		}

		for(i = von + 1; i <= nach; i++)
  	{
			// wenn das feld schon besetzt ist, so drucke coolen kommentar, dass figuren nicht bersprungen werden
			// knnen, und das nur eine figur auf einem feld stehen kann,
			if(Felder[i]->wer() != NIEMAND)
			{
				#ifdef DEBUG
					cout << "Hier steht jemand im Weg" << endl;
				#endif

 				if(!test) zeigeHilfe(HELP_BESETZT);
						
				// es geht nicht
				return -2;
			}
		}
	}
	else // bewegung nach links
	{
		for(i = von - 1; i >= nach; i--)
  	{
			// aufpassen, dass man nicht von rechts nach links luft
			if(i % SPIELFELD_SPALTEN == SPIELFELD_SPALTEN - 1)
			{
				#ifdef DEBUG
					cout << "der zug ist schief" << endl;
				#endif

				if(!test) zeigeHilfe(HELP_SCHIEF);
		
				// der zug ist schief
				return -1;
			}
		}
		for(i = von - 1; i >= nach; i--)
  	{
			// wenn das feld schon besetzt ist, so drucke coolen kommentar, dass figuren nicht bersprungen werden
			// knnen, und das nur eine figur auf einem feld stehen kann,
			if(Felder[i]->wer() != NIEMAND)
			{
				#ifdef DEBUG
					cout << "Hier steht jemand im Weg" << endl;
				#endif

				if(!test) zeigeHilfe(HELP_BESETZT);

		
				// es geht nicht
				return -2;
			}
		}
	}

  if(test)
		return 42;
	
	// waren die felder nicht besetzt, so ziehe

	Felder[von]->setze(NIEMAND);
	Felder[nach]->setze(wer);

	// berprfe die siegbedingungen und schlage figuren

	schlagUndSiege(nach);

	// die situation is 2 mal vorgekommen. jetzt ist das dritte mal

	if(compareWithSnapshots() > 1)
	{
		zeigeHilfe(HELP_3TIMES);

		// sieht nach einem unentschieden aus.
		GameOverText->setText("Spielsituation drei mal\nwiederholt.\nUnentschienden!");
		Fenster.addObject(GameOver);
		amZug = NIEMAND;

    zugNachricht->setText("");

		return 1;
	}


	// archiviere den zug, und berprfe auf unentschieden   (ziemlich nutzlos, wird nicht weiter verwendet)
	Zug zug;
	zug.nach = nach;
	zug.von = von;
	Historie.push_back(zug);

	zuege++;

	// lege eine kopie des spielfeldes in der liste der bisherigen zge ab
	makeSnapshot();


	// wechsele den spieler
	if(amZug == WEISS)
	{
		amZug = SCHWARZ;
		zugNachricht->setText("Am Zug:\n" + nameSchwarz);
	}
	else if(amZug == SCHWARZ)
	{
		amZug = WEISS;
		zugNachricht->setText("Am Zug:\n" + nameWeiss);
	}
	else if(amZug == NIEMAND)
	{
		// spiel zuende, berprfung auf unentschieden sinnlos
		return 1;
	}


	// schau, ob der nchste spieler berhaupt ziehen kann
	if(!zugMoeglich(amZug))
	{
		zeigeHilfe(HELP_NIX_GEHT);
		GameOverText->setText("Kein Zug moeglich.\nUnentschieden!");
		Fenster.addObject(GameOver);
		amZug = NIEMAND;
		zugNachricht->setText("");

		return 1;
	}
	
	
	return 42;
}



int schlagUndSiege(int nr)
{
	int wer = Felder[nr]->wer();

	if(wer == NIEMAND)
	{
		// diese figur kann niemanden schlagen
		return 0;
	}
	
	// berprfe auf knigs-siegbedinung
	if(wer == KOENIG && // nur beim knig wird auch berprft, ob er auf einem randfeld ist
		(nr % SPIELFELD_SPALTEN == 0 || nr % SPIELFELD_SPALTEN == SPIELFELD_SPALTEN - 1
		|| nr / SPIELFELD_SPALTEN == 0 || nr / SPIELFELD_SPALTEN == SPIELFELD_REIHEN - 1))
	{
		// zeige coolen GameOverDialog
		string tmp = nameWeiss + "\nhat gewonnen!";
		GameOverText->setText(tmp.c_str());
		Fenster.addObject(GameOver);
		// lass hilfe text zu spielgewinn anzeigen

		zeigeHilfe(HELP_WIN_WEISS);

		// es spielt niemand mehr
		amZug = NIEMAND;
		zugNachricht->setText("");

		// das schlagen kann man in diesem fall lassen
		return 1;
	}

  // berprfe wer auf den nachbarfeldern steht, und ob er geschlagen wird
  // feld links, es mssen mindistens zwei felder links davon sein
  if(nr % SPIELFELD_SPALTEN > 1)
  {
		if(wer == SCHWARZ)
		{
			// berprfe ob knig geschlagen wird
			if(fangeKoenig(nr - 1))
			{
				schwarzGewinnt();
				return 2;
			}
			// schaue, ob eine weise figur geschlagen werden kann
			else if(Felder[nr - 1]->wer() == WEISS && Felder[nr - 2]->wer() == SCHWARZ)
			{
				Felder[nr - 1]->setze(NIEMAND);
				// hilfe zu figuren schlagen!
				zeigeHilfe(HELP_SCHLAGEN);

				// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
				deleteSnapshots();
			}
		}
		else // eine weisse figur, knig oder normal, niemand wurde schon ausgefiltert
		// es muss nur nach schwarzen figuren gecheckt werden
		if(Felder[nr - 1]->wer() == SCHWARZ && (Felder[nr - 2]->wer() == WEISS || Felder[nr - 2]->wer() == KOENIG))
		{
			Felder[nr - 1]->setze(NIEMAND);
			// hilfe zu figuren schlagen!
			zeigeHilfe(HELP_SCHLAGEN);

			// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
			deleteSnapshots();
		}
	}
	if(nr % SPIELFELD_SPALTEN < SPIELFELD_SPALTEN - 2)
	{
		if(wer == SCHWARZ)
		{
			// berprfe ob knig gesfangen wird
			if(fangeKoenig(nr + 1))
			{
				schwarzGewinnt();
				return 2;
			}
			// schaue, ob eine weise figur geschlagen werden kann
			else if(Felder[nr + 1]->wer() == WEISS && Felder[nr + 2]->wer() == SCHWARZ)
			{
				Felder[nr + 1]->setze(NIEMAND);
				// hilfe zu figuren schlagen!
				zeigeHilfe(HELP_SCHLAGEN);
				
				// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
				deleteSnapshots();
			}
		}
		else // eine weisse figur, knig oder normal, niemand wurde schon ausgefiltert
		// es muss nur nach schwarzen figuren gecheckt werden
		if(Felder[nr + 1]->wer() == SCHWARZ && (Felder[nr + 2]->wer() == WEISS || Felder[nr + 2]->wer() == KOENIG))
		{
			Felder[nr + 1]->setze(NIEMAND);
			// hilfe zu figuren schlagen!
      zeigeHilfe(HELP_SCHLAGEN);
			// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
			deleteSnapshots();
		}
	}
	if(nr / SPIELFELD_SPALTEN > 1)
	{
		if(wer == SCHWARZ)
		{
			// berprfe ob knig geschlagen wird
			if(fangeKoenig(nr - SPIELFELD_SPALTEN))
			{
				schwarzGewinnt();
				return 2;
			}
			// schaue, ob eine weise figur geschlagen werden kann
			else if(Felder[nr - SPIELFELD_SPALTEN]->wer() == WEISS && Felder[nr - 2 * SPIELFELD_SPALTEN]->wer() == SCHWARZ)
			{
				Felder[nr - SPIELFELD_SPALTEN]->setze(NIEMAND);
				// hilfe zu figuren schlagen!
        zeigeHilfe(HELP_SCHLAGEN);
				// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
				deleteSnapshots();
			}
		}
		else // eine weisse figur, knig oder normal, niemand wurde schon ausgefiltert
		// es muss nur nach schwarzen figuren gecheckt werden
		if(Felder[nr - SPIELFELD_SPALTEN]->wer() == SCHWARZ && (Felder[nr - 2 * SPIELFELD_SPALTEN]->wer() == WEISS || Felder[nr - 2 * SPIELFELD_SPALTEN]->wer() == KOENIG))
		{
			Felder[nr - SPIELFELD_SPALTEN]->setze(NIEMAND);
			// hilfe zu figuren schlagen!
      zeigeHilfe(HELP_SCHLAGEN);
			// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
			deleteSnapshots();
		}
	}
	if(nr / SPIELFELD_SPALTEN < SPIELFELD_REIHEN - 2)
	{
		if(wer == SCHWARZ)
		{
			// berprfe ob knig geschlagen wird
			if(fangeKoenig(nr + SPIELFELD_SPALTEN))
			{
				schwarzGewinnt();
				return 2;
			}
			// schaue, ob eine weise figur geschlagen werden kann
			else if(Felder[nr + SPIELFELD_SPALTEN]->wer() == WEISS && Felder[nr + 2 * SPIELFELD_SPALTEN]->wer() == SCHWARZ)
			{
				Felder[nr + SPIELFELD_SPALTEN]->setze(NIEMAND);

				// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
				deleteSnapshots();
				
				// hilfe zu figuren schlagen!
				zeigeHilfe(HELP_SCHLAGEN);
			}
		}
		else // eine weisse figur, knig oder normal, niemand wurde schon ausgefiltert
		// es muss nur nach schwarzen figuren gecheckt werden
		if(Felder[nr + SPIELFELD_SPALTEN]->wer() == SCHWARZ && (Felder[nr + 2 * SPIELFELD_SPALTEN]->wer() == WEISS || Felder[nr + 2 * SPIELFELD_SPALTEN]->wer() == KOENIG))
		{
			Felder[nr + SPIELFELD_SPALTEN]->setze(NIEMAND);
			// hilfe zu figuren schlagen!
      zeigeHilfe(HELP_SCHLAGEN);
			// die alten snapshots kann man jetzt vergessen, da diese spielstnde eh nicht mehr kommen
			deleteSnapshots();
		}
	}
	
	return 0;
}

bool fangeKoenig(int nr)
{
	if(Felder[nr]->wer() != KOENIG)
		return false;

	// auf siegbedingung des knigs muss schon berprft sein!! sonst gibt er sieg fr schwarz aus, wenn knig
	// am rand eingekesselt ist

	// Feld links, aufpassen, dass du nicht rausfliegst, ist es das linkste feld wird der zweite teil der abfrage
	// nicht ausgefhrt (es lebe c++);)
	if(nr % SPIELFELD_SPALTEN > 0
		&& Felder[nr - 1]->wer() != SCHWARZ && nr - 1 != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
		return false;
	// Feld rechts
	if(nr % SPIELFELD_SPALTEN < SPIELFELD_SPALTEN - 1
		&& Felder[nr + 1]->wer() != SCHWARZ && nr + 1 != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
		return false;
	// feld oben
	if(nr / SPIELFELD_SPALTEN > 0
		&& Felder[nr - SPIELFELD_SPALTEN]->wer() != SCHWARZ && nr - SPIELFELD_SPALTEN != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
		return false;
	// Feld rechts
	if(nr / SPIELFELD_SPALTEN < SPIELFELD_REIHEN - 1
		&& Felder[nr + SPIELFELD_SPALTEN]->wer() != SCHWARZ && nr + SPIELFELD_SPALTEN != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2)
		return false;

	// auf allen der umliegenden feldern, die nicht der thron sind, steht ein schwarzer
	return true;
}


void schwarzGewinnt()
{
	string tmp;
	tmp = nameSchwarz + "\nhat gewonnen!";
	GameOverText->setText(tmp);
	Fenster.addObject(GameOver);

	// fieser hilfetext!!!
	zeigeHilfe(HELP_WIN_SCHWARZ);

	amZug = NIEMAND;
	zugNachricht->setText("");
}


bool zugMoeglich(int wer)
{
	for(int i = 0; i < SPIELFELD_SPALTEN * SPIELFELD_REIHEN; i++)
	{
		// nur berprfen, wenn aktuelle figur dem spieler gehrt
		if(wer == Felder[i]->wer() || (wer == WEISS && Felder[i]->wer() == KOENIG))
		{
			// checke feld links, wenn es der thron ist, so muss es der knig sein, der dran ist
			if(i % SPIELFELD_SPALTEN > 0 && Felder[i - 1]->wer() == NIEMAND
				&& (i - 1 != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2 || Felder[i]->wer() == KOENIG))
				return true;
			// feld oben
			if(i / SPIELFELD_SPALTEN > 0 && Felder[i - SPIELFELD_SPALTEN]->wer() == NIEMAND
				&& (i - SPIELFELD_SPALTEN != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2 || Felder[i]->wer() == KOENIG))
				return true;
			// reld rechts
			if(i % SPIELFELD_SPALTEN < SPIELFELD_SPALTEN - 1 && Felder[i + 1]->wer() == NIEMAND
			&& (i + 1 != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2 || Felder[i]->wer() == KOENIG))
				return true;
			// feld unten
			if(i / SPIELFELD_SPALTEN < SPIELFELD_REIHEN - 1 && Felder[i + 1]->wer() == NIEMAND
			&& (i + SPIELFELD_SPALTEN != SPIELFELD_SPALTEN * SPIELFELD_REIHEN / 2 || Felder[i]->wer() == KOENIG))
				return true;
		}
	}
	
  return false;
}

bool pause(int halt)
{
	if(pausiert)
		if(halt == 0 && amZug == NIEMAND)
		{
			amZug = vorPause;
			
			if(amZug == WEISS)
				zugNachricht->setText("Am Zug:\n" + nameWeiss);
			else if(amZug == SCHWARZ)
				zugNachricht->setText("Am Zug:\n" + nameSchwarz);
			
			gewaehlteFigur = vorPauseGewaehlt;
			pausiert = false;
			return false;
		}
		else
		{
			return true;
		}
	else
	{
		if(halt == 1 && amZug != NIEMAND)
		{	
			vorPause = amZug;
			amZug = NIEMAND;
			// whrend der pause ndert sich der ziehende nicht ;)
			vorPauseGewaehlt = gewaehlteFigur;
			pausiert = true;
			return true;
		}
		else  // es luft eh kein spiel
		{
			return false;
		}
	}
}

char dran()
{
	return amZug;
}

int setDran(char wer)
{
	if(pausiert)
		return -1;

	if(wer == SCHWARZ || wer == WEISS)
	{
		amZug = wer;
		return 42;
	}
	else
		return -2;
}

